/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "dialog.h"

static void
destroy_pixmap_callback			(GtkObject	*object,
   					 gpointer	user_data);

GtkWidget* 
dialog_create (gchar *title)
{
  GtkWidget *dialog;
  GtkWidget *dialog_vbox1;
  GtkWidget *dialog_action_area1;
  GtkWidget *hbox1;
  
  dialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dialog), title);
  gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, FALSE, FALSE);
  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);

  dialog_vbox1 = GTK_DIALOG (dialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  dialog_action_area1 = GTK_DIALOG (dialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_container_set_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbox1, TRUE, TRUE, 10);
  
  return dialog;
}

GtkWidget*	dialog_with_label_create	(gchar *title,gchar *message)
{
  GtkWidget	*dialog;
  GtkWidget	*label;
  
  dialog = dialog_create(title);
  
  label = gtk_label_new(message);
  gtk_widget_show(label);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), label, TRUE,TRUE,10);
  
  return dialog;
}

GtkWidget*	dialog_with_pixmap_label_create	(gchar *title,gchar **xpm_data,gchar *message)
{
  GtkWidget	*dialog;
  GtkWidget	*label;
  GtkWidget	*pixmap;
  GdkPixmap	*gdkpixmap;
  GtkStyle	*style;
  GtkWidget	*hbox;
  
  dialog = dialog_create(title);
  gtk_widget_show(dialog);
    
  style = gtk_widget_get_default_style();
  gdkpixmap = gdk_pixmap_create_from_xpm_d(GTK_WIDGET(dialog)->window,NULL,
     					   &(style->bg[GTK_STATE_NORMAL]),(gchar**)xpm_data);
  
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, TRUE,TRUE,10);

  pixmap = gtk_pixmap_new(gdkpixmap,NULL);
  gtk_box_pack_start (GTK_BOX(hbox), pixmap, TRUE,TRUE,10);
  gtk_widget_show(pixmap);
    
  if (message)
  {
    label = gtk_label_new(message);
    gtk_misc_set_alignment(GTK_MISC(label),0.0,0.5);
    gtk_widget_show(label);
    gtk_box_pack_start (GTK_BOX (hbox), label, TRUE,TRUE,10);
  }
  
  gtk_signal_connect(GTK_OBJECT(dialog),"destroy", (GtkSignalFunc)destroy_pixmap_callback,(gpointer)gdkpixmap);
  
  gtk_widget_hide(dialog);
  
  return dialog;
}

GtkWidget*	dialog_with_pixmap_label_entry_create (gchar *title,gchar **xpm_data, gchar* message,GtkWidget **entry)
{
  GtkWidget	*dialog;
  
  dialog = dialog_with_pixmap_label_create(title,xpm_data,message);

  *entry = gtk_entry_new();
  gtk_widget_show(*entry);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), *entry, TRUE,TRUE,10);
  
  return dialog;
}

void dialog_add_button (GtkWidget *dialog,gchar *caption,GtkSignalFunc func,gpointer user_data)
{
  GtkWidget	*button;
  GtkWidget	*hbox;
  GtkBox	*action_area;
  
  action_area = GTK_BOX(GTK_DIALOG(dialog)->action_area);
  hbox = ((GtkBoxChild*)(action_area->children->data))->widget;
  
  button = gtk_button_new_with_label(caption);
  gtk_widget_show(button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (button), 10);
  
  if (func)
    gtk_signal_connect(GTK_OBJECT(button),"clicked",func,user_data);
}

void dialog_add_button_pixmap (GtkWidget *dialog,
   			       gchar** xpm_data,gchar *caption,
                               GtkSignalFunc func,gpointer user_data)
{
  GtkWidget	*button;
  GtkWidget	*pixmap;
  GtkWidget	*label;
  GtkWidget	*hbox;
  GtkBox	*action_area;
  GdkPixmap	*gdkpixmap;
  GtkStyle	*style;
  
  gtk_widget_show(dialog);
  
  style = gtk_widget_get_default_style();
  gdkpixmap = gdk_pixmap_create_from_xpm_d(GTK_WIDGET(dialog)->window,NULL,
     					   &(style->bg[GTK_STATE_NORMAL]),(gchar**)xpm_data);
  
  action_area = GTK_BOX(GTK_DIALOG(dialog)->action_area);
  hbox = ((GtkBoxChild*)(action_area->children->data))->widget;

  button = gtk_button_new();
  gtk_widget_show(button);
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (button), 5);

  pixmap = gtk_pixmap_new(gdkpixmap,NULL);
  gtk_widget_show(pixmap);
  gtk_container_add( GTK_CONTAINER(button), pixmap);

  if (caption)
  {
    label = gtk_label_new(caption);
    gtk_widget_show(label);
    gtk_container_add( GTK_CONTAINER(button), label);
  }
  
  if (func)
    gtk_signal_connect(GTK_OBJECT(button),"clicked",func,user_data);

  gtk_signal_connect(GTK_OBJECT(dialog),"destroy", (GtkSignalFunc)destroy_pixmap_callback,(gpointer)gdkpixmap);
  
  gtk_widget_hide(dialog);
}

void
dialog_destroy_callback			(GtkButton	*button,
   					 gpointer	user_data)
{
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

void
dialog_destroy_gtk_main 		(GtkObject	*object,
   					 gpointer	user_data)
{
  gtk_main_quit();
}

static void
destroy_pixmap_callback			(GtkObject	*object,
   					 gpointer	user_data)
{
  gdk_pixmap_unref((GdkPixmap*)user_data);
}
