/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include "gwf_guiprojio.h"
#include <gwf_lib.h>
#include "fileselect.h"
#include "gwf_message.h"
#include <gtk/gtk.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

typedef struct _GwfIOData	GwfIOData;

struct _GwfIOData
{
  GwfError	*error;
  GwfProjlist	*projlist;
  GwfXmlIO	*xmlio;
};

static void	filesel_save_ok_clicked		(GtkButton *button,gpointer user_data);
static void	filesel_load_ok_clicked		(GtkButton *button,gpointer user_data);
static void	filesel_destroyed		(GtkObject *object,gpointer user_data);

void
gwf_project_save		(GwfError *error,GwfProjlist *projlist,GwfXmlIO *xmlio,GtkWidget **filesel)
{
  GwfProject	*project;

  g_return_if_fail(error != NULL);
  g_return_if_fail(projlist != NULL);
  g_return_if_fail(xmlio != NULL);

  project = gwf_projlist_get_current(projlist);

  if (project && !gwf_project_get_filename(project))
    gwf_project_save_as(error,projlist,xmlio,filesel);
  else
    gwf_save_project(error,projlist,xmlio);
}

void
gwf_project_save_as		(GwfError *error,GwfProjlist *projlist,GwfXmlIO *xmlio,GtkWidget **filesel)
{
  GwfIOData	*savedata;
  gchar		*homedir;

  g_return_if_fail(error != NULL);
  g_return_if_fail(projlist != NULL);
  g_return_if_fail(xmlio != NULL);

  if (!*filesel)
  {
    savedata = g_malloc(sizeof(GwfIOData));
    savedata->error = error;
    savedata->projlist = projlist;
    savedata->xmlio = xmlio;
    *filesel = create_fileselection("Save project as...",GTK_SIGNAL_FUNC(filesel_save_ok_clicked),(gpointer)savedata);
    gtk_signal_connect(GTK_OBJECT(*filesel), "destroy", GTK_SIGNAL_FUNC(filesel_destroyed), (gpointer)savedata);
    homedir = g_strdup_printf("%s%c",g_get_home_dir(),G_DIR_SEPARATOR);
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(*filesel),homedir);
    g_free(homedir);
  }

  gtk_widget_set_sensitive(*filesel,TRUE);
  gtk_widget_show(*filesel);
}

static void
filesel_destroyed(GtkObject *object,gpointer user_data)
{
  g_free(user_data);
}

static void
filesel_save_ok_clicked		(GtkButton *button,gpointer user_data)
{
  GwfIOData		*data = user_data;
  GtkFileSelection	*filesel = fileselection_get(GTK_WIDGET(button));
  FILE			*ifp;
  GwfProject		*project;

  project = gwf_projlist_get_current(data->projlist);
  if (!project)
  {
    gwf_error_raise_error(data->error,"General : No project");
    return;
  }

  gtk_widget_set_sensitive(GTK_WIDGET(filesel),FALSE);

  gwf_project_set_filename(project,gtk_file_selection_get_filename(filesel));

  #ifdef AUTO_FNAME_EXT
   gwf_project_filname_check_extension_add(project);
  #endif

  ifp = fopen(gwf_project_get_filename(project),"r");
  if (ifp)
  {
    fclose(ifp);
    if (gwf_message_choice("File already exists! Overwrite?",GWF_OK | GWF_CANCEL) == GWF_CANCEL)
    {
      gtk_widget_hide(GTK_WIDGET(filesel));
      return;
    }
  }

  gwf_save_project(data->error,data->projlist,data->xmlio);
  gtk_widget_hide(GTK_WIDGET(filesel));
}


void
gwf_project_load		(GwfError *error,GwfProjlist *projlist,GwfXmlIO *xmlio,GtkWidget **filesel)
{
  GwfIOData	*loaddata;
  gchar		*homedir;

  g_return_if_fail(error != NULL);
  g_return_if_fail(projlist != NULL);
  g_return_if_fail(xmlio != NULL);
  g_return_if_fail(filesel != NULL);

  if (!*filesel)  
  {
    loaddata = g_malloc(sizeof(GwfIOData));
    loaddata->error = error;
    loaddata->projlist = projlist;
    loaddata->xmlio = xmlio;
    *filesel = create_fileselection("Load a project",GTK_SIGNAL_FUNC(filesel_load_ok_clicked),(gpointer)loaddata);
    gtk_signal_connect(GTK_OBJECT(*filesel), "destroy", GTK_SIGNAL_FUNC(filesel_destroyed), (gpointer)loaddata);
    homedir = g_strdup_printf("%s%c",g_get_home_dir(),G_DIR_SEPARATOR);
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(*filesel),homedir);
    g_free(homedir);
  }

  gtk_widget_set_sensitive(*filesel,TRUE);
  gtk_widget_show(*filesel);
}

static void
filesel_load_ok_clicked		(GtkButton *button,gpointer user_data)
{
  GwfIOData		*data = user_data;
  GtkFileSelection	*filesel = fileselection_get(GTK_WIDGET(button));
  gchar			*filename;

  filename = gtk_file_selection_get_filename(filesel);
  gtk_widget_set_sensitive(GTK_WIDGET(filesel), FALSE);
  gwf_load_project(data->error,data->projlist,data->xmlio,filename);
  gtk_widget_hide(GTK_WIDGET(filesel));
}

void
gwf_project_write_source	(GwfError *error,GwfProjlist *projlist,GwfSource *source,GwfLicList *liclist)
{
  g_return_if_fail(error != NULL);
  g_return_if_fail(projlist != NULL);
  g_return_if_fail(source != NULL);
  g_return_if_fail(liclist != NULL);

  gwf_write_project_source(error,projlist,source,liclist);
}
