/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "gwf_mainwin.h"
#include <gwf_lib.h>
#include "gwf_window.h"

#include "gwf_aboutbox.h"
#include "gwf_widprop.h"
#include "gwf_projopt.h"
#include "fileselect.h"
#include "gwf_message.h"
#include "gwf_guiprojio.h"
#include "gwf_projtree.h"

static gchar*	windowtitle = "Gtk+ Widget Factory";

static void		gwf_mainwin_class_init		(GwfMainWinClass	*klass);
static void		gwf_mainwin_init		(GwfMainWin		*mainwin);
static void		gwf_mainwin_destroy		(GtkObject		*object);

/*****************************/
/* Mainwindown Functionality */
/*****************************/
static void		gwf_mainwin_new_project		(GwfMainWin *mainwin);
static void		gwf_mainwin_open_project 	(GwfMainWin *mainwin);
static void		gwf_mainwin_save_project 	(GwfMainWin *mainwin);
static void		gwf_mainwin_save_project_as	(GwfMainWin *mainwin);
static void		gwf_mainwin_write_code 		(GwfMainWin *mainwin);
static void		gwf_mainwin_delete_project	(GwfMainWin *mainwin);
static void		gwf_mainwin_show_aboutbox	(GwfMainWin *mainwin);
static gboolean		gwf_mainwin_program_quit_ok 	(GwfMainWin *mainwin);

/*****************/
/* GUI Callbacks */
/*****************/
static void	on_gwfaboutbox_destroy		(GtkObject *object,gpointer user_data);
static gboolean	on_MainWindow_delete_event	(GtkWidget *widget,gpointer user_data);
static void	on_projtree_row_selected	(GtkCTree *ctree,GList *node,gint column,gpointer user_data);

static void	on_file_open_activate		(GtkMenuItem *menuitem,gpointer user_data);
static void	on_file_save_activate		(GtkMenuItem *menuitem,gpointer user_data);
static void	on_file_save_as_activate	(GtkMenuItem *menuitem,gpointer user_data);
static void	on_file_quit_activate		(GtkMenuItem *menuitem,gpointer user_data);
static void	on_edit_cut_activate		(GtkMenuItem *menuitem,gpointer user_data);
static void	on_edit_copy_activate		(GtkMenuItem *menuitem,gpointer user_data);
static void	on_edit_paste_activate		(GtkMenuItem *menuitem,gpointer user_data);
static void	on_widget_new_activate		(GtkMenuItem *menuitem,gpointer user_data);
static void	on_widget_del_activate		(GtkMenuItem *menuitem,gpointer user_data);
static void	on_source_write_activate	(GtkMenuItem *menuitem,gpointer user_data);
static void	on_help_about_activate		(GtkMenuItem *menuitem,gpointer user_data);

static void	on_btnNew_clicked		(GtkButton *button,gpointer user_data);
static void	on_btnOpen_clicked		(GtkButton *button,gpointer user_data);
static void	on_btnSave_clicked		(GtkButton *button,gpointer user_data);
static void	on_btnWriteCode_clicked		(GtkButton *button,gpointer user_data);

/***************************/
/* Window Create Functions */
/***************************/
static void		gwf_mainwin_interface 		(GwfMainWin *mainwin);
static GtkWidget*	gwf_mainwin_menu 		(GwfMainWin *mainwin, GtkAccelGroup *accel_group);
static GtkWidget*	gwf_mainwin_file_menu 		(GwfMainWin *mainwin, GtkAccelGroup *accel_group);
static GtkWidget*	gwf_mainwin_edit_menu 		(GwfMainWin *mainwin, GtkAccelGroup *accel_group);
static GtkWidget*	gwf_mainwin_widget_menu		(GwfMainWin *mainwin, GtkAccelGroup *accel_group);
static GtkWidget*	gwf_mainwin_source_menu		(GwfMainWin *mainwin, GtkAccelGroup *accel_group);
static GtkWidget*	gwf_mainwin_help_menu 		(GwfMainWin *mainwin, GtkAccelGroup *accel_group);
static GtkWidget*	gwf_mainwin_toolbar 		(GwfMainWin *mainwin);

guint
gwf_mainwin_get_type (void)
{
  static guint gwf_mainwin_type = 0;

  if (!gwf_mainwin_type)
    {
      GtkTypeInfo gwf_mainwin_info =
      {
	"GwfMainWin",
	sizeof (GwfMainWin),
	sizeof (GwfMainWinClass),
	(GtkClassInitFunc) gwf_mainwin_class_init,
	(GtkObjectInitFunc) gwf_mainwin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL,
      };

      gwf_mainwin_type = gtk_type_unique (gwf_window_get_type(), &gwf_mainwin_info);
    }

  return gwf_mainwin_type;
}

static void		
gwf_mainwin_class_init (GwfMainWinClass *klass)
{
  ((GtkObjectClass *)klass)->destroy = gwf_mainwin_destroy;
}

static void
gwf_mainwin_init (GwfMainWin *mainwin)
{
}


void
gwf_mainwin_set_licenses (GwfMainWin *mainwin,GwfLicList *liclist)
{
  g_return_if_fail(mainwin != NULL);
  mainwin->liclist = liclist;
}

void
gwf_mainwin_construct(GwfMainWin *mainwin,GwfData *data)
{
  mainwin->settings = data->settings;
  mainwin->treedatas = data->treedatas;
  mainwin->error = data->error;
  mainwin->xmlio = data->xmlio;
  mainwin->plugins = data->plugins;
  mainwin->source = data->source;
  mainwin->liclist = data->liclist;

  gwf_window_construct(GWF_WINDOW(mainwin),data->projlist);
  gwf_mainwin_interface(mainwin);
  gwf_window_set_windowtitle(GWF_WINDOW(mainwin),windowtitle);
  gwf_error_set_func(data->error,(GwfErrorFunc)gwf_errdlg_add_message,(gpointer)mainwin->errdlg);
}

GtkWidget*
gwf_mainwin_new(GwfData *data)
{
  GwfMainWin	*mainwin;

  mainwin = GWF_MAINWIN( gtk_type_new (gwf_mainwin_get_type()));
  gwf_mainwin_construct(mainwin,data);
  return GTK_WIDGET ( mainwin);
}

static void
gwf_mainwin_destroy (GtkObject *object)
{
  GwfMainWin	*mainwin = GWF_MAINWIN(object);
  
  if (mainwin->aboutbox) 
    gtk_widget_destroy(mainwin->aboutbox);

  if (mainwin->load_filesel)
    gtk_widget_destroy(mainwin->load_filesel);

  if (mainwin->save_filesel)
    gtk_widget_destroy(mainwin->save_filesel);
   

  g_free(mainwin->curprojname);

  gtk_signal_disconnect_by_data(GTK_OBJECT(GWF_WINDOW(mainwin)->projlist),(gpointer)object);
  
  gtk_main_quit();
}

/*****************************/
/* Main window Functionality */
/*****************************/
static void
gwf_mainwin_new_project	(GwfMainWin *mainwin)
{
  if (!gwf_projlist_new_unique_project(GWF_WINDOW(mainwin)->projlist))
  {
    gwf_error_raise_error(mainwin->error,"General : Widget exists");
  }
  gwf_error_raise_error(mainwin->error,"General : New project");
}

static void
gwf_mainwin_open_project (GwfMainWin *mainwin)
{
  gwf_project_load(mainwin->error,GWF_WINDOW(mainwin)->projlist,mainwin->xmlio,&mainwin->load_filesel);
}

static void
gwf_mainwin_save_project (GwfMainWin *mainwin)
{
  gwf_project_save(mainwin->error,GWF_WINDOW(mainwin)->projlist,mainwin->xmlio,&mainwin->save_filesel);
}

static void
gwf_mainwin_save_project_as (GwfMainWin *mainwin)
{
  gwf_project_save_as(mainwin->error,GWF_WINDOW(mainwin)->projlist,mainwin->xmlio,&mainwin->save_filesel);
}

static void
gwf_mainwin_write_code (GwfMainWin *mainwin)
{
  gwf_project_write_source(mainwin->error,GWF_WINDOW(mainwin)->projlist,mainwin->source,mainwin->liclist);
}

static void
gwf_mainwin_delete_project(GwfMainWin *mainwin)
{
  GwfProject	*project;
  gchar		*temp;
  
  project = gwf_projlist_get_current(GWF_WINDOW(mainwin)->projlist);

  if (!project)
    return;

  if (gwf_project_get_changed(project) &&
      (gwf_message_choice("Project is not saved! Are you sure?",GWF_YES | GWF_NO) == GWF_NO))
  {
    return;
  }

  temp = g_strdup(gwf_project_get_widname(project));
  gwf_projlist_remove_project(GWF_WINDOW(mainwin)->projlist,temp);
  g_free(temp);

  gwf_error_raise_error(mainwin->error,"General : Project deleted");
}

static void
gwf_mainwin_show_aboutbox(GwfMainWin *mainwin)
{
  if (!mainwin->aboutbox)
  {
    mainwin->aboutbox = gwf_aboutbox_new();
    gtk_signal_connect(GTK_OBJECT(mainwin->aboutbox),"destroy",
       		       (GtkSignalFunc)on_gwfaboutbox_destroy,(gpointer)mainwin);
  }
  
  gwf_window_show(mainwin->aboutbox);
}

static gboolean
gwf_mainwin_program_quit_ok (GwfMainWin *mainwin)
{
  if (gwf_projlist_project_changed(GWF_WINDOW(mainwin)->projlist))
    if (gwf_message_choice ("Some projects are not saved! Quit anyway?",GWF_OK | GWF_CANCEL) == GWF_CANCEL)
      return FALSE;
  
  return TRUE;
}

/****************/
/* GUI Creation */
/****************/
static void
gwf_mainwin_interface (GwfMainWin *mainwin)
{
  GtkWidget 	*vbox;
  GtkWidget	*hbox;
  GtkWidget	*projtree;
  GwfErrdlg	*errdlg;
  GtkWidget	*hpan;
  GtkWidget	*vpan;
  GtkWidget	*widprop;
  GtkWidget	*projopt;

  GtkAccelGroup *accel_group;

  accel_group = gtk_accel_group_new ();

  gwf_window_set_title (GTK_WINDOW (mainwin), windowtitle, NULL);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (mainwin), vbox);

  mainwin->errdlg = errdlg = gwf_errdlg_new();
  gtk_widget_show(GTK_WIDGET(errdlg));

  projtree = gwf_projtree_new(GWF_WINDOW(mainwin)->projlist,mainwin->treedatas);
  gtk_widget_show(projtree);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show(hbox);

  vpan = gtk_vpaned_new();
  gtk_widget_show(vpan);

  gtk_box_pack_start (GTK_BOX (vbox), gwf_mainwin_menu(mainwin,accel_group), FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), gwf_mainwin_toolbar(mainwin), FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), vpan, TRUE, TRUE, 0);

  gtk_paned_pack1(GTK_PANED(vpan),hbox,TRUE,TRUE);
  gtk_paned_pack2(GTK_PANED(vpan),GTK_WIDGET(errdlg),TRUE,TRUE);

  hpan = gtk_hpaned_new();
  gtk_widget_show(hpan);
  gtk_box_pack_start (GTK_BOX (hbox), hpan, TRUE, TRUE, 0);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_widget_show (vbox);

  gtk_paned_pack1(GTK_PANED(hpan),projtree,TRUE,TRUE);
  gtk_paned_pack2(GTK_PANED(hpan),vbox,TRUE,TRUE);

  vpan = gtk_vpaned_new();
  gtk_widget_show(vpan);
  gtk_box_pack_start (GTK_BOX (vbox), vpan, TRUE, TRUE, 0);

  widprop = gwf_widprop_new(GWF_WINDOW(mainwin)->projlist,mainwin->settings);
  gtk_widget_show(widprop);

  projopt = gwf_projopt_new(GWF_WINDOW(mainwin)->projlist,mainwin->liclist);
  gtk_widget_show(projopt);

  gtk_paned_pack1(GTK_PANED(vpan),projopt,TRUE,TRUE);
  gtk_paned_pack2(GTK_PANED(vpan),widprop,TRUE,TRUE);

  gtk_signal_connect (GTK_OBJECT (mainwin), "delete_event",
                      GTK_SIGNAL_FUNC (on_MainWindow_delete_event),
                      (gpointer)mainwin);

  gtk_window_add_accel_group (GTK_WINDOW (mainwin), accel_group);

  mainwin->projopt = GWF_PROJOPT(projopt);
  mainwin->widprop = GWF_WIDPROP(widprop);
  mainwin->projtree = GWF_PROJTREE(projtree);

  gtk_signal_connect (GTK_OBJECT(GWF_PROJTREE(projtree)->projtree), "tree_select_row",
                      GTK_SIGNAL_FUNC(on_projtree_row_selected),
                      (gpointer)mainwin);
}

static void
on_projtree_row_selected(GtkCTree *ctree,GList *node,gint column,gpointer user_data)
{
  GwfMainWin	*mainwin = user_data;
  gchar		*tmp,*label;
  GtkWidget	*page;
  GtkCTreeRow	*row;
  GtkCTreeNode	*nod = GTK_CTREE_NODE(node);
  gint		i=0;

  row = GTK_CTREE_ROW(nod);
  if (!row->parent || !GTK_CTREE_ROW(row->parent)->parent)
    return;

  while (GTK_CTREE_ROW(GTK_CTREE_ROW(row->parent)->parent)->parent)
  {
    nod = row->parent;
    row = GTK_CTREE_ROW(nod);
  }

  if (!gtk_ctree_get_node_info(mainwin->projtree->projtree,GTK_CTREE_NODE(nod),&tmp,NULL,NULL,NULL,NULL,NULL,NULL,NULL))
    return;

  if (!gwf_settingslist_get(mainwin->settings,tmp))
    return;

  do
  {
    page = gtk_notebook_get_nth_page(mainwin->widprop->nb,i);
    if (page)
    {
      gtk_label_get(GTK_LABEL(gtk_notebook_get_tab_label(mainwin->widprop->nb,page)), &label);
      if (strcmp(tmp,label) == 0)
        gtk_notebook_set_page(mainwin->widprop->nb,i);
    }
    i++;
  } while (page != NULL);

}

static void
on_gwfaboutbox_destroy(GtkObject *object,gpointer user_data)
{
  GWF_MAINWIN(user_data)->aboutbox = NULL;
}

static gboolean
on_MainWindow_delete_event		(GtkWidget	*widget,
   					 gpointer	user_data)
{
  return !gwf_mainwin_program_quit_ok(GWF_MAINWIN(widget));
}


/********/
/* Menu */
/********/

static GtkWidget*
gwf_mainwin_menu (GwfMainWin *mainwin, GtkAccelGroup *accel_group)
{
  GtkWidget *menubar;

  menubar = gtk_menu_bar_new ();
  gtk_widget_show (menubar);

  gtk_container_add (GTK_CONTAINER (menubar), gwf_mainwin_file_menu(mainwin,accel_group));
  gtk_container_add (GTK_CONTAINER (menubar), gwf_mainwin_edit_menu(mainwin,accel_group));
  gtk_container_add (GTK_CONTAINER (menubar), gwf_mainwin_widget_menu(mainwin,accel_group));
  gtk_container_add (GTK_CONTAINER (menubar), gwf_mainwin_source_menu(mainwin,accel_group));
  gtk_container_add (GTK_CONTAINER (menubar), gwf_mainwin_help_menu(mainwin,accel_group));
  
  return menubar;
}

/*************/
/* File menu */
/*************/

static GtkWidget*
gwf_mainwin_file_menu (GwfMainWin *mainwin, GtkAccelGroup *accel_group)
{
  GtkWidget *menu;
  GtkWidget *menu_menu;
  GtkAccelGroup *accels;
  GtkWidget *menuitem;
  GtkWidget *separator1;

  menu = gtk_menu_item_new_with_label ("File");
  gtk_widget_show (menu);

  menu_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), menu_menu);
  accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (menu_menu));

  menuitem = gtk_menu_item_new_with_label ("Open...");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_menu), menuitem);
  gtk_widget_add_accelerator (menuitem, "activate", accel_group,
                              GDK_O, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
                      GTK_SIGNAL_FUNC (on_file_open_activate),
                      (gpointer)mainwin);

  menuitem = gtk_menu_item_new_with_label ("Save");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_menu), menuitem);
  gtk_widget_add_accelerator (menuitem, "activate", accel_group,
                              GDK_S, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
                      GTK_SIGNAL_FUNC (on_file_save_activate),
                      (gpointer)mainwin);

  menuitem = gtk_menu_item_new_with_label ("Save as...");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_menu), menuitem);

  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
                      GTK_SIGNAL_FUNC (on_file_save_as_activate),
                      (gpointer)mainwin);

  separator1 = gtk_menu_item_new ();
  gtk_widget_show (separator1);
  gtk_container_add (GTK_CONTAINER (menu_menu), separator1);
  gtk_widget_set_sensitive (separator1, FALSE);


  menuitem = gtk_menu_item_new_with_label ("Quit");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_menu), menuitem);
  gtk_widget_add_accelerator (menuitem, "activate", accel_group,
                              GDK_Q, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);
                              
  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
                      GTK_SIGNAL_FUNC (on_file_quit_activate),
                      (gpointer)mainwin);

  return menu;

}

static void
on_file_open_activate			(GtkMenuItem	*menuitem,
   					 gpointer	user_data)
{
  gwf_mainwin_open_project(GWF_MAINWIN(user_data));
}

static void
on_file_save_activate			(GtkMenuItem	*menuitem,
   					 gpointer	user_data)
{
  gwf_mainwin_save_project(GWF_MAINWIN(user_data));
}

static void
on_file_save_as_activate		(GtkMenuItem	*menuitem,
   					 gpointer	user_data)
{
  gwf_mainwin_save_project_as(GWF_MAINWIN(user_data));
}

static void
on_file_quit_activate			(GtkMenuItem	*menuitem,
   					 gpointer	user_data)
{
  if (gwf_mainwin_program_quit_ok(GWF_MAINWIN(user_data)))
    gtk_widget_destroy(GTK_WIDGET(user_data));
}

/*************/
/* Edit menu */
/*************/

static GtkWidget*
gwf_mainwin_edit_menu (GwfMainWin *mainwin, GtkAccelGroup *accel_group)
{
  GtkWidget *menu;
  GtkWidget *menu_menu;
  GtkAccelGroup *accels;
  GtkWidget *menuitem;

  menu = gtk_menu_item_new_with_label ("Edit");
  gtk_widget_show (menu);

  /* FIXME: Until cut & paste is ready */
  gtk_widget_set_sensitive(menu,FALSE);
  /* */

  menu_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), menu_menu);
  accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (menu_menu));

  menuitem = gtk_menu_item_new_with_label ("Cut");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_menu), menuitem);
  gtk_widget_add_accelerator (menuitem, "activate", accel_group,
                              GDK_X, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
                      GTK_SIGNAL_FUNC (on_edit_cut_activate),
                      (gpointer)mainwin);

  menuitem = gtk_menu_item_new_with_label ("Copy");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_menu), menuitem);
  gtk_widget_add_accelerator (menuitem, "activate", accel_group,
                              GDK_C, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
                      GTK_SIGNAL_FUNC (on_edit_copy_activate),
                      (gpointer)mainwin);

  menuitem = gtk_menu_item_new_with_label ("Paste");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_menu), menuitem);
  gtk_widget_add_accelerator (menuitem, "activate", accel_group,
                              GDK_V, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
                      GTK_SIGNAL_FUNC (on_edit_paste_activate),
                      (gpointer)mainwin);

  return menu;
}

static void
on_edit_cut_activate			(GtkMenuItem	*menuitem,
   					 gpointer	user_data)
{
  /* FIXME: cut */
}

static void
on_edit_copy_activate			(GtkMenuItem	*menuitem,
   					 gpointer	user_data)
{
  /* FIXME: copy */
}

static void
on_edit_paste_activate			(GtkMenuItem	*menuitem,
   					 gpointer	user_data)
{
  /* FIXME: paste */
}

/***************/
/* Widget menu */
/***************/

static GtkWidget*
gwf_mainwin_widget_menu (GwfMainWin *mainwin, GtkAccelGroup *accel_group)
{
  GtkWidget *menu;
  GtkWidget *menu_menu;
  GtkAccelGroup *accels;
  GtkWidget *menuitem;

  menu = gtk_menu_item_new_with_label ("Widget");
  gtk_widget_show (menu);

  menu_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), menu_menu);
  accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (menu_menu));

  menuitem = gtk_menu_item_new_with_label ("New widget/object");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_menu), menuitem);

  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
                      GTK_SIGNAL_FUNC (on_widget_new_activate),
                      (gpointer)mainwin);

  menuitem = gtk_menu_item_new_with_label ("Delete widget/object...");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_menu), menuitem);

  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
                      GTK_SIGNAL_FUNC (on_widget_del_activate),
                      (gpointer)mainwin);

  return menu;
}

static void
on_widget_new_activate			(GtkMenuItem	*menuitem,
   					 gpointer	user_data)
{
  gwf_mainwin_new_project(GWF_MAINWIN(user_data));
}

static void
on_widget_del_activate			(GtkMenuItem	*menuitem,
   					 gpointer	user_data)
{
  gwf_mainwin_delete_project(GWF_MAINWIN(user_data));
}

/***************/
/* Source menu */
/***************/

static GtkWidget*
gwf_mainwin_source_menu (GwfMainWin *mainwin, GtkAccelGroup *accel_group)
{
  GtkWidget *menu;
  GtkWidget *menu_menu;
  GtkAccelGroup *accels;
  GtkWidget *menuitem;

  menu = gtk_menu_item_new_with_label ("Source");
  gtk_widget_show (menu);

  menu_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), menu_menu);
  accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (menu_menu));

  menuitem = gtk_menu_item_new_with_label ("Write code");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_menu), menuitem);
  gtk_widget_add_accelerator (menuitem, "activate", accel_group,
                              GDK_W, GDK_CONTROL_MASK,
                              GTK_ACCEL_VISIBLE);

  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
                      GTK_SIGNAL_FUNC (on_source_write_activate),
                      (gpointer)mainwin);

  return menu;
}

static void
on_source_write_activate		(GtkMenuItem	*menuitem,
   					 gpointer	user_data)
{
  gwf_mainwin_write_code(GWF_MAINWIN(user_data));
}


/*************/
/* Help menu */
/*************/

static GtkWidget*
gwf_mainwin_help_menu (GwfMainWin *mainwin, GtkAccelGroup *accel_group)
{
  GtkWidget *menu;
  GtkWidget *menu_menu;
  GtkAccelGroup *accels;
  GtkWidget *menuitem;

  menu = gtk_menu_item_new_with_label ("Help");
  gtk_widget_show (menu);
  gtk_menu_item_right_justify (GTK_MENU_ITEM (menu));

  menu_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), menu_menu);
  accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (menu_menu));

  menuitem = gtk_menu_item_new_with_label ("About...");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu_menu), menuitem);
  
  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
                      GTK_SIGNAL_FUNC (on_help_about_activate),
                      (gpointer)mainwin);

  return menu;
}

static void
on_help_about_activate			(GtkMenuItem	*menuitem,
   					 gpointer	user_data)
{
  gwf_mainwin_show_aboutbox(GWF_MAINWIN(user_data));
}

/***********/
/* Toolbar */
/***********/

static GtkWidget*
gwf_mainwin_toolbar (GwfMainWin *mainwin)
{
  GtkWidget *toolbar1;
  GtkWidget *tmp_toolbar_icon;
  GtkWidget *btnNew;
  GtkWidget *btnOpen;
  GtkWidget *btnSave;
  GtkWidget *btnWriteCode;

  toolbar1 = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
  gtk_widget_show (toolbar1);

  tmp_toolbar_icon = gwf_pixmap_create(GTK_WIDGET(mainwin),new_xpm);
  btnNew = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "",
                                "New Project", NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_show (btnNew);

  tmp_toolbar_icon = gwf_pixmap_create (GTK_WIDGET(mainwin), open_xpm);
  btnOpen = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "",
                                "Open Project", NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_show (btnOpen);

  tmp_toolbar_icon = gwf_pixmap_create (GTK_WIDGET(mainwin), save_xpm);
  btnSave = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "",
                                "Save Project", NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_show (btnSave);

  tmp_toolbar_icon = gwf_pixmap_create (GTK_WIDGET(mainwin), source_xpm);
  btnWriteCode = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "",
                                "Write Source Code", NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_show (btnWriteCode);

  gtk_signal_connect (GTK_OBJECT (btnNew), "clicked",
                      GTK_SIGNAL_FUNC (on_btnNew_clicked),
                      (gpointer)mainwin);
  gtk_signal_connect (GTK_OBJECT (btnOpen), "clicked",
                      GTK_SIGNAL_FUNC (on_btnOpen_clicked),
                      (gpointer)mainwin);
  gtk_signal_connect (GTK_OBJECT (btnSave), "clicked",
                      GTK_SIGNAL_FUNC (on_btnSave_clicked),
                      (gpointer)mainwin);
  gtk_signal_connect (GTK_OBJECT (btnWriteCode), "clicked",
                      GTK_SIGNAL_FUNC (on_btnWriteCode_clicked),
                      (gpointer)mainwin);

  return toolbar1;
}

static void
on_btnNew_clicked			(GtkButton	*button,
   					 gpointer	user_data)
{
  gwf_mainwin_new_project(GWF_MAINWIN(user_data));
}

static void
on_btnOpen_clicked			(GtkButton	*button,
   					 gpointer	user_data)
{
  gwf_mainwin_open_project(GWF_MAINWIN(user_data));
}

static void
on_btnSave_clicked			(GtkButton	*button,
   					 gpointer	user_data)
{
  gwf_mainwin_save_project(GWF_MAINWIN(user_data));
}

static void
on_btnWriteCode_clicked			(GtkButton	*button,
   					 gpointer	user_data)
{
  gwf_mainwin_write_code(GWF_MAINWIN(user_data));
}
