/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include "gwf_widprop.h"
#include <gwf_lib.h>

static void		gwf_widprop_class_init		(GwfWidPropClass	*klass);
static void		gwf_widprop_init		(GwfWidProp		*widprop);
static void		gwf_widprop_destroy		(GtkObject 		*object);

/****************/
/* GUI Creation */
/****************/
static void		gwf_widprop_interface 		(GwfWidProp *widprop);
static void		gwf_widprop_settings 		(GwfWidProp *widprop);
static void		gwf_widprop_add_setting 	(GwfSettings *settings,gpointer user_data);
static void		gwf_widprop_add_nb_settings	(GwfWidProp *widprop,GwfSettings *settings);

guint
gwf_widprop_get_type (void)
{
  static guint gwf_widprop_type = 0;

  if (!gwf_widprop_type)
    {
      GtkTypeInfo gwf_widprop_info =
      {
	"GwfWidProp",
	sizeof (GwfWidProp),
	sizeof (GwfWidPropClass),
	(GtkClassInitFunc) gwf_widprop_class_init,
	(GtkObjectInitFunc) gwf_widprop_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL,
      };

      gwf_widprop_type = gtk_type_unique (gtk_vbox_get_type(), &gwf_widprop_info);
    }

  return gwf_widprop_type;
}

static void		
gwf_widprop_class_init (GwfWidPropClass *klass)
{
  ((GtkObjectClass *)klass)->destroy = gwf_widprop_destroy;
}

static void
gwf_widprop_init (GwfWidProp *widprop)
{
  widprop->pages = 0;
  widprop->basicdone = FALSE;
  gwf_widprop_interface(widprop);
}

void
gwf_widprop_construct (GwfWidProp *widprop,GwfProjlist *projlist,GwfSettingsList *list)
{
  GwfSettings	*settings;

  widprop->projlist = projlist;
  widprop->list = list;
  settings = gwf_settingslist_get(widprop->list,"Basic");
  if (settings)
    gwf_widprop_add_setting(settings,(gpointer)widprop);
  widprop->basicdone = TRUE;

  gwf_widprop_settings(widprop);
}

GtkWidget*
gwf_widprop_new	(GwfProjlist *projlist,GwfSettingsList *list)
{
  GwfWidProp	*widprop;
  
  widprop = GWF_WIDPROP (gtk_type_new (gwf_widprop_get_type()));
  gwf_widprop_construct(widprop,projlist,list);
  return GTK_WIDGET ( widprop);
}

static void
gwf_widprop_destroy (GtkObject *object)
{
}

/****************/
/* GUI Creation */
/****************/

static void
gwf_widprop_interface (GwfWidProp *widprop)
{
  GtkWidget	*notebook;

  notebook = gtk_notebook_new ();
  widprop->nb = GTK_NOTEBOOK(notebook);
  gtk_widget_show (notebook);
  gtk_box_pack_start (GTK_BOX (widprop), notebook, TRUE, TRUE, 0);
}

static void
gwf_widprop_settings (GwfWidProp *widprop)
{
  gwf_settingslist_foreach(widprop->list,gwf_widprop_add_setting,(gpointer)widprop);
}

static void
gwf_widprop_add_setting (GwfSettings *settings,gpointer user_data)
{
  if ((strcmp (settings->name,"Basic") != 0) || !GWF_WIDPROP(user_data)->basicdone)
  {
    gwf_widprop_add_nb_settings(GWF_WIDPROP(user_data),settings);
  }
}

static void
gwf_widprop_add_nb_settings(GwfWidProp *widprop,GwfSettings *settings)
{
  GtkWidget	*label;
  GtkWidget	*setting;
  
  setting = gwf_settings_create(settings,widprop->projlist);
  if (setting)
  {
    gtk_widget_show(setting);
    gtk_container_add (GTK_CONTAINER(widprop->nb), setting);
    label = gtk_label_new (settings->name);
    gtk_widget_show (label);
    gtk_notebook_set_tab_label (widprop->nb, gtk_notebook_get_nth_page (widprop->nb, widprop->pages), label);
    widprop->pages++;
  }
}
