/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "gwf_license.h"
#include <gtk/gtk.h>

typedef struct _GwfLicListForeach	GwfLicListForeach;

struct _GwfLicListForeach
{
  GwfLicenseForeachFunc	func;
  gpointer		user_data;
};

static gboolean		free_license_foreach		(gpointer key,gpointer value,gpointer user_data);
static void		foreach_license			(gpointer key,gpointer value,gpointer user_data);



GwfLicense*
gwf_license_new (gchar *name)
{
   GwfLicense	*tmp;

   g_return_val_if_fail(name != NULL,NULL);

   tmp = g_malloc(sizeof(GwfLicense));

   tmp->name = g_strdup(name);
   tmp->text = NULL;

   return tmp;
}

void
gwf_license_free (GwfLicense *license)
{
  g_return_if_fail(license != NULL);

  g_free(license->name);
  g_free(license->text);
  g_free(license);
}

void
gwf_license_set_text (GwfLicense *license,gchar *txt)
{
  g_return_if_fail(license != NULL);

  g_free(license->text);
  license->text = g_strdup(txt);
}


GwfLicList*
gwf_liclist_new	(void)
{
  GwfLicList		*tmp;

  tmp = g_malloc(sizeof(GwfLicList));

  tmp->licenses = g_hash_table_new(g_str_hash,g_str_equal);
  tmp->std = NULL;

  return tmp;
}

void
gwf_liclist_free (GwfLicList *liclist)
{
  g_return_if_fail(liclist != NULL);

  g_free(liclist->std);
  g_hash_table_foreach_remove(liclist->licenses,free_license_foreach,NULL);
  g_free(liclist);
}

void
gwf_liclist_insert (GwfLicList *liclist,GwfLicense *license)
{
  g_return_if_fail(liclist != NULL);
  g_return_if_fail(license != NULL);

  if (gwf_liclist_get(liclist,license->name))
    return;

  g_hash_table_insert(liclist->licenses,(gpointer)license->name,(gpointer)license);
}

void
gwf_liclist_remove (GwfLicList *liclist,gchar *name)
{
  GwfLicense	*tmp;

  g_return_if_fail(liclist != NULL);
  g_return_if_fail(name != NULL);

  tmp = gwf_liclist_get(liclist,name);
  if (!tmp)
    return;

  g_hash_table_remove(liclist->licenses,name);

  gwf_license_free(tmp);
}

GwfLicense*
gwf_liclist_get (GwfLicList *liclist,gchar *name)
{
  g_return_val_if_fail(liclist != NULL,NULL);
  g_return_val_if_fail(name != NULL,NULL);

  return (GwfLicense*)g_hash_table_lookup(liclist->licenses,name);
}

gint
gwf_liclist_set_std (GwfLicList *liclist,gchar *name)
{
  g_return_val_if_fail(liclist != NULL,0);
  g_return_val_if_fail(name != NULL,0);

  if (!gwf_liclist_get(liclist,name))
    return 0;

  g_free(liclist->std);
  liclist->std = g_strdup(name);

  return 1;
}

void
gwf_liclist_foreach (GwfLicList *liclist,GwfLicenseForeachFunc func,gpointer user_data)
{
  GwfLicListForeach	data;

  g_return_if_fail(liclist != NULL);
  g_return_if_fail(func != NULL);

  data.func = func;
  data.user_data = user_data;

  g_hash_table_foreach(liclist->licenses,foreach_license,&data);
}



static void
foreach_license (gpointer key,gpointer value,gpointer user_data)
{
  GwfLicListForeach	*data = user_data;

  data->func((GwfLicense*)value,data->user_data);
}

static gboolean
free_license_foreach (gpointer key,gpointer value,gpointer user_data)
{
  gwf_license_free((GwfLicense*)value);

  return TRUE;
}
