/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "gwf_projdata.h"
#include <gtk/gtk.h>

typedef struct _GwfDataForeach		GwfDataForeach;

struct _GwfDataForeach
{
  GwfDataForeachFunc	func;
  gpointer		user_data;
};

static gboolean	gwf_data_hash_free_elem	(gpointer key,gpointer value,gpointer user_data);
static void	foreach_data		(gpointer key,gpointer value,gpointer user_data);

GwfDataList*
gwf_data_list_new (void)
{
  GwfDataList		*tmp;
   
  tmp = g_malloc(sizeof(GwfDataList));
   
  tmp->data = g_hash_table_new(g_str_hash,g_str_equal);
  
  return tmp;
}

void
gwf_data_list_free (GwfDataList *list)
{
  g_return_if_fail(list != NULL);
  
  g_hash_table_foreach_remove(list->data,gwf_data_hash_free_elem,NULL);
  g_hash_table_destroy(list->data);
  g_free(list);
}

gint
gwf_data_list_insert (GwfDataList *list,GwfProjData *data)
{
  g_return_val_if_fail(list != NULL,0);
  g_return_val_if_fail(data != NULL,0);
  
  if (g_hash_table_lookup(list->data,(gconstpointer)data->name))
    return 0;
    
  g_hash_table_insert(list->data,data->name,(gpointer)data);
  return 1;  
}

void
gwf_data_list_remove (GwfDataList *list,gchar *name)
{
  GwfProjData	*tmp;
  
  g_return_if_fail(list != NULL);
  g_return_if_fail(name != NULL);
  
  tmp = g_hash_table_lookup(list->data,(gconstpointer)name);
  if (!tmp)
    return;
    
  g_hash_table_remove(list->data,(gconstpointer)name);
  if (tmp->free_func) tmp->free_func(tmp);
  g_free(tmp->name);
  g_free(tmp);
}

GwfProjData*
gwf_data_list_get (GwfDataList *list,gchar *name)
{
  g_return_val_if_fail(list != NULL,NULL);
  g_return_val_if_fail(name != NULL,NULL);
  
  return g_hash_table_lookup(list->data,(gconstpointer)name);
}

gpointer
gwf_data_list_get_data (GwfDataList *list,gchar *name)
{
  GwfProjData	*tmp;
  
  g_return_val_if_fail(list != NULL,NULL);
  g_return_val_if_fail(name != NULL,NULL);
  
  tmp = g_hash_table_lookup(list->data,(gconstpointer)name);
  if (!tmp)
    return NULL;
    
  return tmp->data;
}

void
gwf_data_list_foreach (GwfDataList *list,GwfDataForeachFunc func,gpointer user_data)
{
  GwfDataForeach	tmp;
  g_return_if_fail(list != NULL);
  g_return_if_fail(func != NULL);
  
  tmp.func = func;
  tmp.user_data = user_data;
  
  g_hash_table_foreach(list->data,foreach_data,(gpointer)&tmp);
}

GwfProjData*
gwf_proj_data_new (gchar *name)
{
  GwfProjData	*tmp;
  
  g_return_val_if_fail(name != NULL,NULL);
  
  tmp = g_malloc(sizeof(GwfProjData));
  tmp->name = g_strdup(name);
  tmp->active = TRUE;
  tmp->data = NULL;
  tmp->free_func = NULL;
  
  return tmp;
}

void
gwf_proj_data_free (GwfProjData *data)
{
  g_return_if_fail(data != NULL);
  
  g_free(data->name);
  if (data->free_func) data->free_func(data->data);
  g_free(data);
}

void
gwf_proj_data_set_data (GwfProjData *data,gpointer dat)
{
  g_return_if_fail(data != NULL);
  
  data->data = dat;
}

void
gwf_proj_data_set_free_func (GwfProjData *data,GwfDataFreeFunc func)
{
  g_return_if_fail(data != NULL);
  
  data->free_func = func;
}

gpointer
gwf_proj_data_get_data (GwfProjData *data)
{
  g_return_val_if_fail(data != NULL,NULL);
  
  return data->data;
}

void
gwf_proj_data_set_active (GwfProjData *data,gboolean active)
{
  g_return_if_fail(data != NULL);
  
  data->active = active;
}

gboolean
gwf_proj_data_get_active (GwfProjData *data)
{
  g_return_val_if_fail(data != NULL,FALSE);
  
  return data->active;
}



static gboolean
gwf_data_hash_free_elem	(gpointer key,gpointer value,gpointer user_data)
{
  gwf_proj_data_free((GwfProjData *)value);
  
  return TRUE;
}

static void
foreach_data(gpointer key,gpointer value,gpointer user_data)
{
  GwfDataForeach	*tmp = user_data;
  
  tmp->func((GwfProjData*)value,tmp->user_data);
}


