/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include "gwf_project.h"
#include "gwf_projdata.h"
#include "gwf_gpl.h"
#include "gwf_tools.h"

enum {
   AUTHOR_CHANGE,
   EMAIL_CHANGE,
   FILENAME_CHANGE,
   CFILE_CHANGE,
   HFILE_CHANGE,
   WIDNAME_CHANGE,
   PRNTNAME_CHANGE,
   PRNTINCL_CHANGE,
   BACKUP_CHANGE,
   EXAMPLE_CHANGE,
   LICENSE_CHANGE,
   WIDGET_TYPE_CHANGE,

   PLUG_ACT_TOGGLE,
   PROJECT_CHANGE,
   
   LAST_SIGNAL
};

static void		gwf_project_class_init		(GwfProjectClass	*klass);
static void		gwf_project_init		(GwfProject		*project);
static void		gwf_project_destroy		(GtkObject		*object);

static GtkObjectClass *parent_class = NULL;
static guint project_signals[LAST_SIGNAL] = { 0 };

guint
gwf_project_get_type (void)
{
  static guint gwf_project_type = 0;

  if (!gwf_project_type)
    {
      GtkTypeInfo gwf_project_info =
      {
	"GwfProject",
	sizeof (GwfProject),
	sizeof (GwfProjectClass),
	(GtkClassInitFunc) gwf_project_class_init,
	(GtkObjectInitFunc) gwf_project_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL,
      };

      gwf_project_type = gtk_type_unique (gtk_object_get_type (),
					  &gwf_project_info);
    }

  return gwf_project_type;
}

static void		
gwf_project_class_init (GwfProjectClass *klass)
{
  GtkObjectClass *object_class;

  object_class = (GtkObjectClass *) klass;

  parent_class = gtk_type_class (gtk_object_get_type ());
  
  object_class->destroy = gwf_project_destroy;

  project_signals[AUTHOR_CHANGE] =
    gtk_signal_new ("author_changed",
                    GTK_RUN_FIRST,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GwfProjectClass, author_changed),
                    gtk_marshal_NONE__STRING,
                    GTK_TYPE_NONE,1,
                    GTK_TYPE_STRING);
  project_signals[EMAIL_CHANGE] =
    gtk_signal_new ("email_changed",
                    GTK_RUN_FIRST,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GwfProjectClass, email_changed),
                    gtk_marshal_NONE__STRING,
                    GTK_TYPE_NONE,1,
                    GTK_TYPE_STRING);
  project_signals[FILENAME_CHANGE] = 
    gtk_signal_new ("filename_changed",
       		     GTK_RUN_FIRST,
                     object_class->type,
                     GTK_SIGNAL_OFFSET (GwfProjectClass, filename_changed),
                     gtk_marshal_NONE__STRING,
                     GTK_TYPE_NONE,1,
                     GTK_TYPE_STRING);
  project_signals[CFILE_CHANGE] =
    gtk_signal_new ("cfile_changed",
       		    GTK_RUN_FIRST,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GwfProjectClass, cfile_changed),
                    gtk_marshal_NONE__STRING,
                    GTK_TYPE_NONE,1,
                    GTK_TYPE_STRING);
  project_signals[HFILE_CHANGE] =
    gtk_signal_new ("hfile_changed",
       		    GTK_RUN_FIRST,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GwfProjectClass, hfile_changed),
                    gtk_marshal_NONE__STRING,
                    GTK_TYPE_NONE,1,
                    GTK_TYPE_STRING);
  project_signals[WIDNAME_CHANGE] = 
    gtk_signal_new ("widname_changed",
                    GTK_RUN_FIRST,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GwfProjectClass, widname_changed),
                    gtk_marshal_NONE__STRING,
                    GTK_TYPE_NONE,1,
                    GTK_TYPE_STRING);
  project_signals[PRNTNAME_CHANGE] = 
    gtk_signal_new ("prntname_changed",
                    GTK_RUN_FIRST,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GwfProjectClass, prntname_changed),
                    gtk_marshal_NONE__STRING,
                    GTK_TYPE_NONE,1,
                    GTK_TYPE_STRING);
  project_signals[PRNTINCL_CHANGE] =
    gtk_signal_new ("prntincl_changed",
                    GTK_RUN_FIRST,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GwfProjectClass, prntincl_changed),
                    gtk_marshal_NONE__STRING,
                    GTK_TYPE_NONE,1,
                    GTK_TYPE_STRING);
  project_signals[BACKUP_CHANGE] =
    gtk_signal_new ("backup_changed",
                    GTK_RUN_FIRST,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GwfProjectClass, backup_changed),
                    gtk_marshal_NONE__BOOL,
                    GTK_TYPE_NONE,1,
                    GTK_TYPE_BOOL);
  project_signals[EXAMPLE_CHANGE] = 
    gtk_signal_new ("example_changed",
                    GTK_RUN_FIRST,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GwfProjectClass, example_changed),
                    gtk_marshal_NONE__BOOL,
                    GTK_TYPE_NONE,1,
                    GTK_TYPE_BOOL);
  project_signals[LICENSE_CHANGE] =
    gtk_signal_new ("license_changed",
                    GTK_RUN_FIRST,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GwfProjectClass, license_changed),
                    gtk_marshal_NONE__STRING,
                    GTK_TYPE_NONE,1,
                    GTK_TYPE_STRING);
  project_signals[WIDGET_TYPE_CHANGE] =
    gtk_signal_new ("widget_type_changed",
                    GTK_RUN_FIRST,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GwfProjectClass, widget_type_changed),
                    gtk_marshal_NONE__STRING,
                    GTK_TYPE_NONE,1,
                    GTK_TYPE_STRING);

  project_signals[PLUG_ACT_TOGGLE] =
    gtk_signal_new ("plugin_active_toggled",
                    GTK_RUN_FIRST,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GwfProjectClass, plugin_active_toggle),
                    gtk_marshal_NONE__STRING,
                    GTK_TYPE_NONE,1,
                    GTK_TYPE_STRING);
  project_signals[PROJECT_CHANGE] =
    gtk_signal_new ("project_changed",
                    GTK_RUN_FIRST,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GwfProjectClass, project_changed),
                    gtk_marshal_NONE__NONE,
                    GTK_TYPE_NONE,0);

  gtk_object_class_add_signals (object_class, project_signals, LAST_SIGNAL);
  
  klass->author_changed = NULL;
  klass->email_changed = NULL;
  klass->filename_changed = NULL;
  klass->cfile_changed = NULL;
  klass->hfile_changed = NULL;
  klass->widname_changed = NULL;
  klass->prntname_changed = NULL;
  klass->prntincl_changed = NULL;
  klass->backup_changed = NULL;
  klass->example_changed = NULL;
  klass->license_changed = NULL;
  klass->widget_type_changed = NULL;

  klass->plugin_active_toggle = NULL;
  klass->project_changed = NULL;
}


static void
gwf_project_init (GwfProject * project)
{
  project->author = g_strdup( (EMPTY_TO_NULL(g_get_real_name())) ? g_get_real_name() : g_get_user_name());
  project->email = NULL;
  
  project->filename = NULL;
  project->changed = TRUE;
  
  project->c_file = g_strdup("widget.c");
  project->h_file = g_strdup("widget.h");
  
  project->widname = NULL;

  project->prntname = NULL;
  project->prntincl = NULL;

  project->backup = TRUE;
  project->example = FALSE;
  
  project->license = g_strdup(gpl_name);
  
  project->proj_data = gwf_data_list_new();
}

GwfProject*
gwf_project_new(void)
{
  return GWF_PROJECT( gtk_type_new (gwf_project_get_type()));
}

static void
gwf_project_destroy (GtkObject *object)
{
  GwfProject 	*project;
  
  project = GWF_PROJECT (object);
  
  g_free(project->author);
  g_free(project->email);
  
  g_free(project->filename);
  
  g_free(project->c_file);
  g_free(project->h_file);
  
  g_free(project->widname);

  g_free(project->prntname);
  g_free(project->prntincl);

  g_free(project->license);

  gwf_data_list_free(project->proj_data);
  
  parent_class->destroy(object);
}

/*************************/
/* Set and Get functions */
/*************************/

void
gwf_project_set_author (GwfProject *project, gchar *author)
{
  g_return_if_fail(project != NULL);

  g_free(project->author);
  project->author = g_strdup(author);
  gtk_signal_emit(GTK_OBJECT(project),project_signals[AUTHOR_CHANGE],author);
  gwf_project_set_changed(project,TRUE);
}

gchar*
gwf_project_get_author(GwfProject *project)
{
  g_return_val_if_fail(project != NULL, NULL);
  
  return project->author;
}


void
gwf_project_set_email (GwfProject *project, gchar *email)
{
  g_return_if_fail(project != NULL);
  
  g_free(project->email);
  project->email = g_strdup(email);
  gtk_signal_emit(GTK_OBJECT(project),project_signals[EMAIL_CHANGE],email);
  gwf_project_set_changed(project,TRUE);
}

gchar*
gwf_project_get_email (GwfProject *project)
{
  g_return_val_if_fail(project != NULL,NULL);
  
  return project->email;
}


void
gwf_project_set_filename (GwfProject *project, gchar *filename)
{
  g_return_if_fail(project != NULL);
  
  g_free(project->filename);
  project->filename = g_strdup(filename);
  gtk_signal_emit(GTK_OBJECT(project),project_signals[FILENAME_CHANGE],filename);
  gwf_project_set_changed(project,TRUE);
}

gchar*
gwf_project_get_filename (GwfProject *project)
{
  g_return_val_if_fail(project != NULL,NULL);
  
  return project->filename;
}


void
gwf_project_set_changed (GwfProject *project, gboolean changed)
{
  gboolean	temp;
  g_return_if_fail(project != NULL);
  
  temp = project->changed;
  
  project->changed = changed;
  
  if (changed != temp)
    gtk_signal_emit(GTK_OBJECT(project),project_signals[PROJECT_CHANGE]);
}

gboolean
gwf_project_get_changed (GwfProject *project)
{
  g_return_val_if_fail(project != NULL,FALSE);
  
  return project->changed;
}


void
gwf_project_set_cfile (GwfProject *project, gchar *c_file)
{
  gint		len;
  gchar		*tmp = NULL;
  
  g_return_if_fail(project != NULL);
  
  if (c_file)
  {
    c_file = g_strchug(g_strdup(c_file));
    tmp = c_file;
    len = strlen(c_file);
    if (strcmp( &c_file[len-2] ,".c") != 0)
    {
      c_file = g_strdup_printf("%s.c",c_file);
      g_free(tmp);
      tmp = c_file;
    }
  }
  g_free(project->c_file);
  project->c_file = g_strdup(c_file);
  gtk_signal_emit(GTK_OBJECT(project),project_signals[CFILE_CHANGE],c_file);
  gwf_project_set_changed(project,TRUE);
  g_free(tmp);
}

gchar*
gwf_project_get_cfile (GwfProject *project)
{
  g_return_val_if_fail(project != NULL,NULL);
  
  return project->c_file;
}


void
gwf_project_set_hfile (GwfProject *project, gchar *h_file)
{
  gint		len;
  gchar		*tmp = NULL;

  g_return_if_fail(project != NULL);
  
  if (h_file)
  {
    h_file = g_strchug(g_strdup(h_file));
    tmp = h_file;
    len = strlen(h_file);
    if (strcmp( &h_file[len-2] ,".h") != 0)
    {
      h_file = g_strdup_printf("%s.h",h_file);
      g_free(tmp);
      tmp = h_file;
    }
  }
  g_free(project->h_file);
  project->h_file = g_strdup(h_file);
  gtk_signal_emit(GTK_OBJECT(project),project_signals[HFILE_CHANGE],h_file);
  gwf_project_set_changed(project,TRUE);
  g_free(tmp);
}

gchar*
gwf_project_get_hfile (GwfProject *project)
{
  g_return_val_if_fail(project != NULL,NULL);
  
  return project->h_file;
}


void
gwf_project_set_widname (GwfProject *project,gchar *widname)
{
  g_return_if_fail(project != NULL);
  
  g_free(project->widname);
  project->widname = g_strdup(widname);
  gtk_signal_emit(GTK_OBJECT(project),project_signals[WIDNAME_CHANGE],widname);
  gwf_project_set_changed(project,TRUE);
}

gchar*
gwf_project_get_widname (GwfProject *project)
{
  g_return_val_if_fail(project != NULL,NULL);
  
  return project->widname;
}


void
gwf_project_set_prntname (GwfProject *project,gchar *prntname)
{
  g_return_if_fail(project != NULL);
  
  g_free(project->prntname);
  project->prntname = g_strdup(prntname);
  gtk_signal_emit(GTK_OBJECT(project),project_signals[PRNTNAME_CHANGE],prntname);
  gwf_project_set_changed(project,TRUE);
}

gchar*
gwf_project_get_prntname (GwfProject *project)
{
  g_return_val_if_fail(project != NULL,NULL);
  
  return project->prntname;
}

void
gwf_project_set_prntincl (GwfProject *project,gchar *prntincl)
{
  g_return_if_fail(project != NULL);

  g_free(project->prntincl);
  project->prntincl = g_strdup(prntincl);
  gtk_signal_emit(GTK_OBJECT(project),project_signals[PRNTINCL_CHANGE],prntincl);
  gwf_project_set_changed(project,TRUE);
}

gchar*
gwf_project_get_prntincl (GwfProject *project)
{
  g_return_val_if_fail(project != NULL,NULL);

  return project->prntincl;
}

void
gwf_project_set_backup (GwfProject *project,gboolean backup)
{
  g_return_if_fail(project != NULL);
  
  project->backup = backup;
  gtk_signal_emit(GTK_OBJECT(project),project_signals[BACKUP_CHANGE],backup);
  gwf_project_set_changed(project,TRUE);
}

gboolean
gwf_project_get_backup (GwfProject *project)
{
  g_return_val_if_fail(project != NULL,FALSE);
  
  return project->backup;
}


void
gwf_project_set_example (GwfProject *project,gboolean example)
{
  g_return_if_fail(project != NULL);
  
  project->example = example;
  gtk_signal_emit(GTK_OBJECT(project),project_signals[EXAMPLE_CHANGE],example);
  gwf_project_set_changed(project,TRUE);
}

gboolean
gwf_project_get_example (GwfProject *project)
{
  g_return_val_if_fail(project != NULL,FALSE);
  
  return project->example;
}

void
gwf_project_set_license (GwfProject *project,gchar *license)
{
  g_return_if_fail(project != NULL);
  
  g_free(project->license);
  project->license = g_strdup(license);
  gtk_signal_emit(GTK_OBJECT(project),project_signals[LICENSE_CHANGE],license);
  gwf_project_set_changed(project,TRUE);
}

gchar*
gwf_project_get_license (GwfProject *project)
{
  g_return_val_if_fail(project != NULL,NULL);
  
  return project->license;
}

void
gwf_project_set_widget_type (GwfProject *project,gchar *type)
{
  g_return_if_fail(project != NULL);
  
  g_free(project->widget_type);
  project->widget_type = g_strdup(type);
  gtk_signal_emit(GTK_OBJECT(project),project_signals[WIDGET_TYPE_CHANGE],type);
  gwf_project_set_changed(project,TRUE);
}

gchar*
gwf_project_get_widget_type (GwfProject *project)
{
  g_return_val_if_fail(project != NULL,NULL);
  
  return project->widget_type;
}

/****************************/
/* Plugin support functions */
/****************************/

void
gwf_project_plugin_set_active (GwfProject *project,gchar *name,gboolean active)
{
  GwfProjData	*data;

  g_return_if_fail(project != NULL);
  g_return_if_fail(name != NULL);

  data = gwf_data_list_get(project->proj_data,name);
  g_return_if_fail(data != NULL);

  if (data->active != active)
  {
    data->active = active;

    gtk_signal_emit(GTK_OBJECT(project),project_signals[PLUG_ACT_TOGGLE],data->name);
    gwf_project_set_changed(project,TRUE);
  }
}

gboolean
gwf_project_plugin_get_active (GwfProject *project,gchar *name)
{
  GwfProjData	*data;

  g_return_val_if_fail(project != NULL,FALSE);
  g_return_val_if_fail(name != NULL,FALSE);

  data = gwf_data_list_get(project->proj_data,name);
  g_return_val_if_fail(data != NULL,FALSE);

  return data->active;
}


GwfDataList*
gwf_project_get_data_list (GwfProject *project)
{
  g_return_val_if_fail(project != NULL,NULL);
  
  return project->proj_data;
}

guint
gwf_project_new_signal (GwfProject *project,  gchar *name,   GtkSignalMarshaller marshaller,
                        GtkType	return_val,   guint nparams, ...)
{
  va_list	args;
  gint		ret;

  g_return_val_if_fail(project != NULL,0);
  g_return_val_if_fail(name != NULL,0);
  g_return_val_if_fail(marshaller != NULL,0);

  va_start(args,nparams);
  ret = gtk_object_class_user_signal_newv(GTK_OBJECT(project)->klass,
     					   name,
                                           GTK_RUN_FIRST,
                                           marshaller,
                                           return_val,
                                           nparams,
                                           args);
  va_end(args);

  return ret;
}
