/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef GWF_PROJECT_H
#define GWF_PROJECT_H

#ifdef __cplusplus
extern "C" {
#endif

#include <gtk/gtk.h>
#include "gwf_projdata.h"

#define GWF_PROJECT(obj)		GTK_CHECK_CAST(obj, gwf_project_get_type (), GwfProject)
#define GWF_PROJECT_CLASS(klass)	GTK_CHECK_CLASS_CAST(klass, gwf_project_get_type (), GwfProjectClass)
#define GWF_IS_PROJECT(obj)		GTK_CHECK_TYPE(obj, gwf_project_get_type())

typedef struct _GwfProject 		GwfProject;
typedef struct _GwfProjectClass		GwfProjectClass;

struct _GwfProject
{
  GtkObject	parent;
  
  gchar		*author;
  gchar		*email;
  
  gchar		*filename;
  gboolean	changed;
  
  gchar		*c_file;
  gchar		*h_file;
  
  gchar		*widname;
  gchar		*widget_type;

  gboolean	backup;

  gchar		*license;

  gchar		*prntname;
  gchar		*prntincl;

  GwfDataList	*proj_data;

/* Will be handled by plugins. For now handled by the project itself until plugin support is ready. */

  gboolean	example;
};

struct _GwfProjectClass
{
  GtkObjectClass	parent_class;
  
  void	(*author_changed)	(GwfProject *project, gchar *author);
  void	(*email_changed)	(GwfProject *project, gchar *email);
  void	(*filename_changed)	(GwfProject *project, gchar *filename);
  void	(*cfile_changed)	(GwfProject *project, gchar *c_file);
  void	(*hfile_changed)	(GwfProject *project, gchar *h_file);
  void	(*widname_changed)	(GwfProject *project, gchar *widname);
  void	(*prntname_changed)	(GwfProject *project, gchar *prntname);
  void	(*prntincl_changed)	(GwfProject *project, gchar *prntincl);
  void	(*backup_changed)	(GwfProject *project, gboolean backup);
  void	(*example_changed)	(GwfProject *project, gboolean example);
  void	(*license_changed)	(GwfProject *project, gchar *license);
  void	(*widget_type_changed)	(GwfProject *project, gchar *type);

  void	(*plugin_active_toggle)	(GwfProject *project, gchar *name);
  void	(*project_changed)	(GwfProject *project);
};

guint		gwf_project_get_type 		(void);
GwfProject*	gwf_project_new			(void);

void		gwf_project_set_author 		(GwfProject *project, gchar *author);
gchar*		gwf_project_get_author		(GwfProject *project);

void		gwf_project_set_email 		(GwfProject *project, gchar *email);
gchar*		gwf_project_get_email 		(GwfProject *project);

void		gwf_project_set_filename 	(GwfProject *project, gchar *filename);
gchar*		gwf_project_get_filename 	(GwfProject *project);

void		gwf_project_set_changed 	(GwfProject *project, gboolean changed);
gboolean	gwf_project_get_changed 	(GwfProject *project);

void		gwf_project_set_cfile 		(GwfProject *project, gchar *c_file);
gchar*		gwf_project_get_cfile 		(GwfProject *project);

void		gwf_project_set_hfile 		(GwfProject *project, gchar *h_file);
gchar*		gwf_project_get_hfile 		(GwfProject *project);

void		gwf_project_set_widname 	(GwfProject *project,gchar *widname);
gchar*		gwf_project_get_widname 	(GwfProject *project);

void		gwf_project_set_prntname 	(GwfProject *project,gchar *prntname);
gchar*		gwf_project_get_prntname 	(GwfProject *project);

void		gwf_project_set_prntincl 	(GwfProject *project,gchar *prntincl);
gchar*		gwf_project_get_prntincl 	(GwfProject *project);

void		gwf_project_set_backup		(GwfProject *project,gboolean backup);
gboolean	gwf_project_get_backup		(GwfProject *project);

void		gwf_project_set_example		(GwfProject *project,gboolean example);
gboolean	gwf_project_get_example		(GwfProject *project);

void		gwf_project_set_license		(GwfProject *project,gchar *license);
gchar*		gwf_project_get_license		(GwfProject *project);

void		gwf_project_set_widget_type	(GwfProject *project,gchar *type);
gchar*		gwf_project_get_widget_type	(GwfProject *project);


/***********/
/* PLUGINS */
/***********/

void		gwf_project_plugin_set_active	(GwfProject *project,gchar *name,gboolean active);
gboolean	gwf_project_plugin_get_active	(GwfProject *project,gchar *name);

GwfDataList*	gwf_project_get_data_list	(GwfProject		*project);
guint		gwf_project_new_signal		(GwfProject 		*project,
   						 gchar 			*name,
                                                 GtkSignalMarshaller	marshaller,
                                                 GtkType		return_val,
                                                 guint			nparams,
                                                 ...);

/***********/

#ifdef __cplusplus
}
#endif

#endif

