/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include "gwf_widpropbasic.h"
#include "gwf_projlist.h"
#include "gwf_project.h"
#include "gwf_tools.h"

static void		gwf_widpropbasic_class_init		(GwfWidPropBasicClass	*klass);
static void		gwf_widpropbasic_init			(GwfWidPropBasic	*wpbas);
static void		gwf_widpropbasic_destroy		(GtkObject		*object);

static void		gwf_widpropbasic_update			(GwfWidPropBasic	*wpbas);
static void		gwf_widpropbasic_connect		(GwfProject *project,GwfWidPropBasic *wpbas);
static void		gwf_widpropbasic_disconnect		(GwfProject *project,GwfWidPropBasic *wpbas);

/************************************/
/* Project & Project list callbacks */
/************************************/
static void curproj_changed_cb		(GwfProjlist *projlist, gchar *name,gpointer user_data);
static void project_added_cb		(GwfProjlist *projlist, gchar *name,gpointer user_data);
static void widname_changed_cb		(GwfProject *project, gchar *widname,gpointer user_data);
static void prntname_changed_cb		(GwfProject *project, gchar *prntname,gpointer user_data);
static void prntincl_changed_cb		(GwfProject *project, gchar *prntincl,gpointer user_data);
static void widgettype_changed_cb	(GwfProject *project, gchar *widgettype,gpointer user_data);

/*****************/
/* GUI Callbacks */
/*****************/
static void	on_btnParentBrowse_clicked		(GtkButton *button,gpointer user_data);
static gboolean	on_enName_focus_out_event		(GtkWidget *widget,GdkEventFocus *event,gpointer user_data);
static gboolean	on_enParent_focus_out_event		(GtkWidget *widget,GdkEventFocus *event,gpointer user_data);
static gboolean	on_enPrntincl_focus_out_event		(GtkWidget *widget,GdkEventFocus *event,gpointer user_data);
static gboolean	on_enWidgetType_focus_out_event		(GtkWidget *widget,GdkEventFocus *event,gpointer user_data);

/****************/
/* GUI Creation */
/****************/
static void		gwf_widpropbasic_interface 	(GwfWidPropBasic *wpbas);


guint
gwf_widpropbasic_get_type (void)
{
  static guint gwf_widpropbasic_type = 0;

  if (!gwf_widpropbasic_type)
    {
      GtkTypeInfo gwf_widpropbasic_info =
      {
	"GwfWidPropBasic",
	sizeof (GwfWidPropBasic),
	sizeof (GwfWidPropBasicClass),
	(GtkClassInitFunc) gwf_widpropbasic_class_init,
	(GtkObjectInitFunc) gwf_widpropbasic_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL,
      };

      gwf_widpropbasic_type = gtk_type_unique (gtk_vbox_get_type(), &gwf_widpropbasic_info);
    }

  return gwf_widpropbasic_type;
}

static void		
gwf_widpropbasic_class_init (GwfWidPropBasicClass *klass)
{
  ((GtkObjectClass *)klass)->destroy = gwf_widpropbasic_destroy;
}

static void
gwf_widpropbasic_init (GwfWidPropBasic *wpbas)
{
  gwf_widpropbasic_interface(wpbas);
}

void
gwf_widpropbasic_construct(GwfWidPropBasic *wpbas,GwfProjlist *projlist)
{
  wpbas->projlist = projlist;
  gwf_projlist_project_foreach(projlist,(GwfProjlistForeachFunc)gwf_widpropbasic_connect,(gpointer)wpbas);
  
  gtk_signal_connect (GTK_OBJECT (projlist), "curproj_changed",
                      GTK_SIGNAL_FUNC(curproj_changed_cb),
                      (gpointer)wpbas);  
  gtk_signal_connect (GTK_OBJECT (projlist), "project_added",
                      GTK_SIGNAL_FUNC (project_added_cb),
                      (gpointer)wpbas);
                      
  gwf_widpropbasic_update(wpbas);
}

GtkWidget *
gwf_widpropbasic_new(GwfProjlist *projlist)
{
  GwfWidPropBasic	*wpbas;
  
  wpbas = GWF_WIDPROP_BASIC(gtk_type_new (gwf_widpropbasic_get_type()));
  gwf_widpropbasic_construct(wpbas,projlist);  
  return GTK_WIDGET ( wpbas);
}

static void
gwf_widpropbasic_destroy (GtkObject *object)
{
  gwf_projlist_project_foreach(GWF_WIDPROP_BASIC(object)->projlist,
     			       (GwfProjlistForeachFunc)gwf_widpropbasic_disconnect,(gpointer)object);
  gtk_signal_disconnect_by_data(GTK_OBJECT(GWF_WIDPROP_BASIC(object)->projlist),(gpointer)object);

}




static void
gwf_widpropbasic_connect (GwfProject *project,GwfWidPropBasic *wpbas)
{
  gtk_signal_connect (GTK_OBJECT (project), "widname_changed",
     		      GTK_SIGNAL_FUNC (widname_changed_cb),
                      (gpointer)wpbas);
  gtk_signal_connect (GTK_OBJECT (project), "prntname_changed",
     		      GTK_SIGNAL_FUNC (prntname_changed_cb),
                      (gpointer)wpbas);
  gtk_signal_connect (GTK_OBJECT (project), "prntincl_changed",
     		      GTK_SIGNAL_FUNC (prntincl_changed_cb),
                      (gpointer)wpbas);
  gtk_signal_connect (GTK_OBJECT (project), "widget_type_changed",
     		      GTK_SIGNAL_FUNC (widgettype_changed_cb),
                      (gpointer)wpbas);
}

static void
gwf_widpropbasic_disconnect (GwfProject *project,GwfWidPropBasic *wpbas)
{
  gtk_signal_disconnect_by_data(GTK_OBJECT(project),(gpointer)wpbas);
}

static void
gwf_widpropbasic_update		(GwfWidPropBasic *wpbas)
{
  GwfProject	*project;
  
  project = gwf_projlist_get_current(wpbas->projlist);
  if (!project)
  {
    gtk_entry_set_text(wpbas->name,"");
    gtk_entry_set_text(wpbas->parent,"");
    gtk_entry_set_text(wpbas->prntincl,"");
    gtk_entry_set_text(wpbas->widgettype,"");
  }
  else
  {
    gtk_entry_set_text(wpbas->name,NULL_TO_EMPTY(gwf_project_get_widname(project)));
    gtk_entry_set_text(wpbas->parent,NULL_TO_EMPTY(gwf_project_get_prntname(project)));
    gtk_entry_set_text(wpbas->prntincl,NULL_TO_EMPTY(gwf_project_get_prntincl(project)));
    gtk_entry_set_text(wpbas->widgettype,NULL_TO_EMPTY(gwf_project_get_widget_type(project)));
  }
}

/************************************/
/* Project & Project list callbacks */
/************************************/

static void 
curproj_changed_cb (GwfProjlist *projlist, gchar *name,gpointer user_data)
{
  gwf_widpropbasic_update(GWF_WIDPROP_BASIC(user_data));
}

static void
project_added_cb	(GwfProjlist *projlist, gchar *name,gpointer user_data)
{
  GwfProject	*project;

  project = gwf_projlist_get_project(projlist,name);
  gwf_widpropbasic_connect(project,GWF_WIDPROP_BASIC(user_data));
}

static void	
widname_changed_cb	(GwfProject *project, gchar *widname,gpointer user_data)
{
  GwfWidPropBasic	*wpbas = user_data;
  
  if (!gwf_projlist_get_current(wpbas->projlist))
    return;

  gtk_entry_set_text(wpbas->name,NULL_TO_EMPTY(widname));
}

static void
prntname_changed_cb	(GwfProject *project, gchar *prntname,gpointer user_data)
{
  GwfWidPropBasic	*wpbas = user_data;
  
  if (!gwf_projlist_get_current(wpbas->projlist))
    return;

  gtk_entry_set_text(wpbas->parent,NULL_TO_EMPTY(prntname));
}

static void
prntincl_changed_cb	(GwfProject *project, gchar *prntincl,gpointer user_data)
{
  GwfWidPropBasic	*wpbas = user_data;

  if (!gwf_projlist_get_current(wpbas->projlist))
    return;

  gtk_entry_set_text(wpbas->prntincl,NULL_TO_EMPTY(prntincl));
}

static void
widgettype_changed_cb (GwfProject *project, gchar *widgettype,gpointer user_data)
{
  GwfWidPropBasic	*wpbas = user_data;
  
  if (!gwf_projlist_get_current(wpbas->projlist))
    return;

  gtk_entry_set_text(wpbas->widgettype,NULL_TO_EMPTY(widgettype));
}

/*****************/
/* GUI Callbacks */
/*****************/
static void
on_btnParentBrowse_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
  /* FIXME: Parent browser */
}

static gboolean
on_enName_focus_out_event              (GtkWidget       *widget,
                                        GdkEventFocus   *event,
                                        gpointer         user_data)
{
  GwfWidPropBasic	*wpbas = user_data;
  gchar			*temp = NULL;
  GwfProject		*project;
  
  project = gwf_projlist_get_current(wpbas->projlist);
  if (!project)
    return FALSE;
    
  temp = gtk_editable_get_chars(GTK_EDITABLE(widget),0,-1);

  if (!gwf_projlist_project_change_name(wpbas->projlist,gwf_project_get_widname(project),temp))
    gtk_entry_set_text(GTK_ENTRY(widget),gwf_project_get_widname(project));
 
  g_free(temp);
  
  return FALSE;
}


static gboolean
on_enParent_focus_out_event            (GtkWidget       *widget,
                                        GdkEventFocus   *event,
                                        gpointer         user_data)
{
  GwfWidPropBasic	*wpbas = user_data;
  gchar			*temp = NULL;
  GwfProject		*project;
  
  project = gwf_projlist_get_current(wpbas->projlist);
  if (!project)
    return FALSE;

  temp = gtk_editable_get_chars(GTK_EDITABLE(widget),0,-1);
  gwf_project_set_prntname(project,EMPTY_TO_NULL(temp));
  g_free(temp);

  return FALSE;
}

static gboolean
on_enPrntincl_focus_out_event          (GtkWidget       *widget,
                                        GdkEventFocus   *event,
                                        gpointer         user_data)
{
  GwfWidPropBasic	*wpbas = user_data;
  gchar			*temp = NULL;
  GwfProject		*project;

  project = gwf_projlist_get_current(wpbas->projlist);
  if (!project)
    return FALSE;

  temp = gtk_editable_get_chars(GTK_EDITABLE(widget),0,-1);
  gwf_project_set_prntincl(project,EMPTY_TO_NULL(temp));
  g_free(temp);

  return FALSE;
}

static gboolean
on_enWidgetType_focus_out_event        (GtkWidget       *widget,
                                        GdkEventFocus   *event,
                                        gpointer         user_data)
{
  GwfWidPropBasic	*wpbas = user_data;
  gchar			*temp = NULL;
  GwfProject		*project;
  
  project = gwf_projlist_get_current(wpbas->projlist);
  if (!project)
    return FALSE;

  temp = gtk_editable_get_chars(GTK_EDITABLE(widget),0,-1);
  gwf_project_set_widget_type(project,EMPTY_TO_NULL(temp));
  g_free(temp);

  return FALSE;
}

/****************/
/* GUI Creation */
/****************/
static void
gwf_widpropbasic_interface (GwfWidPropBasic *wpbas)
{
  GtkWidget	*table;
  GtkWidget	*label;
  GtkWidget	*entry;
  GtkWidget	*checkbutton;
  GtkWidget	*combo;
  GtkWidget	*button;
  GList		*items = NULL;
  
  table = gtk_table_new (3, 7, FALSE);
  gtk_widget_show (table);
  gtk_box_pack_start(GTK_BOX(wpbas),table,TRUE,TRUE,0);

  label = gtk_label_new ("Name :");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  entry = gtk_entry_new ();
  wpbas->name = GTK_ENTRY(entry);
  gtk_widget_show (entry);
  gtk_table_attach (GTK_TABLE (table), entry, 1, 3, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);

  gtk_signal_connect (GTK_OBJECT (entry), "focus_out_event",
                      GTK_SIGNAL_FUNC (on_enName_focus_out_event),
                      (gpointer)wpbas);

  label = gtk_label_new ("Parent :");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  entry = gtk_entry_new ();
  wpbas->parent = GTK_ENTRY(entry);
  gtk_widget_show (entry);
  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);

  gtk_signal_connect (GTK_OBJECT (entry), "focus_out_event",
                      GTK_SIGNAL_FUNC (on_enParent_focus_out_event),
                      (gpointer)wpbas);

  button = gtk_button_new_with_label ("Browse tree...");
  gtk_widget_show (button);
  gtk_table_attach (GTK_TABLE (table), button, 2, 3, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 5, 5);

  /* FIXME: Until parent browser is ready*/
  gtk_widget_set_sensitive(button,FALSE);
  /* */

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_btnParentBrowse_clicked),
                      (gpointer)wpbas);

  label = gtk_label_new ("Parent Include :");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

  entry = gtk_entry_new ();
  wpbas->prntincl = GTK_ENTRY(entry);
  gtk_widget_show (entry);
  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);

  gtk_signal_connect (GTK_OBJECT (entry), "focus_out_event",
                      GTK_SIGNAL_FUNC (on_enPrntincl_focus_out_event),
                      (gpointer)wpbas);

  label = gtk_label_new ("Widget Type :");
  gtk_widget_show(label);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  
  /* FIXME: Until widget type is ready*/
  gtk_widget_set_sensitive(label,FALSE);
  /* */

  combo = gtk_combo_new();
  wpbas->widgettype = GTK_ENTRY(GTK_COMBO(combo)->entry);
  gtk_widget_show (combo);
  items = g_list_append(items,"Gtk+");
  items = g_list_append(items,"Gnome");
  gtk_combo_set_popdown_strings(GTK_COMBO(combo),items);
  g_list_free(items);
  gtk_table_attach (GTK_TABLE (table), combo, 1, 3, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 5, 5);

  /* FIXME: Until widget type is ready*/
  gtk_widget_set_sensitive(combo,FALSE);
  gtk_widget_set_sensitive(GTK_COMBO(combo)->entry,FALSE);
  /* */

  gtk_signal_connect (GTK_OBJECT (GTK_COMBO(combo)->entry), "focus_out_event",
                      GTK_SIGNAL_FUNC (on_enWidgetType_focus_out_event),
                      (gpointer)wpbas);
}
