/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include <gwf_lib.h>
#include "gwf_projtree_signal.h"
#include "gwf_projdata_signal.h"

enum
{
  SIGNAL_ADD,
  SIGNAL_REMOVE,

  LAST_NODE
};

enum
{
  SIGNAL_NAME,
  SIGNAL_RETVAL,
  SIGNAL_RVTYPE,
  SIGNAL_PARAM_ADD,
  SIGNAL_PARAM_REMOVE,

  LAST_SIGNAL_NODE
};

enum
{
  PARAM_NAME,
  PARAM_TYPE,
  PARAM_GTKTYPE,

  LAST_PARAM_NODE
};

typedef struct _SignalAddData	SignalAddData;
typedef struct _ParamAddData	ParamAddData;

struct _SignalAddData
{
  GwfProject	*project;
  GwfPTData	*ptdata;
};

struct _ParamAddData
{
  GwfProject	*project;
  gchar		*signame;
  GwfPTData	*ptdata;
};

static void		signal_tree_add_signal			(GwfProject *project,gchar *name,GwfPTData *ptdata);
static void		signal_tree_remove_signal		(GwfProject *project,gchar *name,GwfPTData *ptdata);

static void		signal_tree_add_foreach_signal		(offersig *signal,gpointer user_data);
static void		signal_tree_add_foreach_param		(sigparam *param,gpointer user_data);

static void		signal_tree_param_added_cb		(GwfProject *project,gchar *signame,gchar *name,GwfPTData *ptdata);
static void		signal_tree_param_removed_cb		(GwfProject *project,gchar *signame,gchar *name,GwfPTData *ptdata);

static void		signal_tree_retval_changed_cb 		(GwfProject *project,gchar *name,gpointer user_data);
static void		signal_tree_rvtype_changed_cb 		(GwfProject *project,gchar *name,gpointer user_data);
static void		signal_tree_name_changed_cb 		(GwfProject *project,gchar *name,gpointer user_data);

static void		signal_tree_param_name_changed_cb 	(GwfProject *project,gchar *signame,gchar *name,gpointer user_data);
static void		signal_tree_param_type_changed_cb 	(GwfProject *project,gchar *signame,gchar *name,gpointer user_data);
static void		signal_tree_param_gtktype_changed_cb 	(GwfProject *project,gchar *signame,gchar *name,gpointer user_data);

static gint		signal_tree_check_current_signal	(GwfProject *project,gchar *signame,GwfPTData *ptdata);
static gint		signal_tree_check_current_param_signal	(GwfProject *project,gchar *signame,gchar *name,GwfPTData *ptdata);


static GwfPTData		sigmain_pt_data[LAST_NODE] =
  {
    {NULL,	"Signals",	NULL,	"offer_signal_added",	0,	signal_tree_add_signal,		NULL, NULL},
    {NULL,	"Signals",	NULL,	"offer_signal_removed",	0,	signal_tree_remove_signal,	NULL, NULL}
  };

static GwfPTData		signal_pt_data[LAST_SIGNAL_NODE] =
  {
    {NULL,	"Name",		NULL,	"offer_signal_changed",	0,	signal_tree_name_changed_cb,	NULL, NULL},
    {NULL,	"Return value",	NULL,	"offer_signal_changed", 0,	signal_tree_retval_changed_cb,	NULL, NULL},
    {NULL,	"Return Type",	NULL,	"offer_signal_changed", 0,	signal_tree_rvtype_changed_cb,	NULL, NULL},
    {NULL,	"Parameters",	NULL,	"ofrsig_param_added",	0,	signal_tree_param_added_cb,	NULL, NULL},
    {NULL,	"Parameters",	NULL,	"ofrsig_param_removed",	0,	signal_tree_param_removed_cb,	NULL, NULL}
  };

static GwfPTData		param_pt_data[LAST_PARAM_NODE] =
  {
    {NULL,	"Name",		NULL,	"ofrsig_param_changed",	0,	signal_tree_param_name_changed_cb,	NULL, NULL},
    {NULL,	"Type",		NULL,	"ofrsig_param_changed", 0,	signal_tree_param_type_changed_cb,	NULL, NULL},
    {NULL,	"GtkType",	NULL,	"ofrsig_param_changed", 0,	signal_tree_param_gtktype_changed_cb,	NULL, NULL}
  };

void
gwf_projtree_signal_attach (GwfTreeData *data,GwfProject *project)
{
  GwfPTData	**pt_data;
  SignalAddData	adddata;

  g_return_if_fail(data != NULL);
  g_return_if_fail(project != NULL);
  g_return_if_fail(data->tree != NULL);

  pt_data = gwf_projtree_add_new_section(data,project,sigmain_pt_data[SIGNAL_ADD].node,sigmain_pt_data,LAST_NODE);
  if (!pt_data)
    return;

  adddata.project = project;
  adddata.ptdata = pt_data[SIGNAL_ADD];
  gwf_project_ofrsig_foreach(project,signal_tree_add_foreach_signal,(gpointer)&adddata);
}

static void
signal_tree_add_foreach_signal(offersig *signal,gpointer user_data)
{
  g_return_if_fail(signal != NULL);
  g_return_if_fail(user_data != NULL);

  signal_tree_add_signal(((SignalAddData*)user_data)->project,signal->name,((SignalAddData*)user_data)->ptdata);
}

static void
signal_tree_add_signal(GwfProject *project,gchar *name,GwfPTData *ptdata)
{
  GtkCTreeNode	*signal;
  GwfPTData	**pt_data;
  ParamAddData	adddata;
  offersig	*sig;

  sig = gwf_project_ofrsig_find(project,name);
  g_return_if_fail(sig != NULL);

  signal = gtk_ctree_add_node(ptdata->data->tree,ptdata->parent,"Signal");
  if (!signal)
    return;
  gtk_ctree_node_set_row_data(ptdata->data->tree,signal,(gpointer)sig);
  gtk_ctree_expand(ptdata->data->tree,signal);

  pt_data = gwf_treedata_register_ptdata(ptdata->data,signal_pt_data,LAST_SIGNAL_NODE,signal,project);
  if (!pt_data)
    return;

  gwf_projtree_insert_string_by_ptdata(pt_data[SIGNAL_NAME],name);
  gwf_projtree_insert_string_by_ptdata(pt_data[SIGNAL_RETVAL],gwf_project_ofrsig_get_retval(project,name));
  gwf_projtree_insert_string_by_ptdata(pt_data[SIGNAL_RVTYPE],gtk_type_to_string(gwf_project_ofrsig_get_rvtype(project,name)));
  gtk_ctree_add_node(ptdata->data->tree,signal,signal_pt_data[SIGNAL_PARAM_ADD].node);

  adddata.project = project;
  adddata.signame = name;
  adddata.ptdata = pt_data[SIGNAL_PARAM_ADD];
  gwf_project_ofrsig_param_foreach(project,name,signal_tree_add_foreach_param,(gpointer)&adddata);
}

static void
signal_tree_remove_signal(GwfProject *project,gchar *name,GwfPTData *ptdata)
{
  offersig	*sig;
  GtkCTreeNode	*signal;

  sig = gwf_project_ofrsig_find(project,name);
  g_return_if_fail(sig != NULL);

  signal = gtk_ctree_find_by_row_data(ptdata->data->tree,ptdata->parent,(gpointer)sig);
  g_return_if_fail(signal != NULL);

  gwf_projtree_remove_node_ptdata(ptdata->data,signal);
}

static void
signal_tree_add_foreach_param(sigparam *param,gpointer user_data)
{
  ParamAddData	*data = user_data;

  g_return_if_fail(param != NULL);
  g_return_if_fail(user_data != NULL);

  signal_tree_param_added_cb(data->project,data->signame,param->name,data->ptdata);
}

static void
signal_tree_param_added_cb(GwfProject *project,gchar *signame,gchar *name,GwfPTData *ptdata)
{
  GtkCTreeNode	*paramnode;
  GtkCTreeNode	*parent;
  GwfPTData	**pt_data;
  sigparam	*param;

  if (!signal_tree_check_current_signal(project,signame,ptdata))
    return;

  param = gwf_project_ofrsig_param_find(project,signame,name);
  g_return_if_fail(param != NULL);

  parent = gtk_ctree_find_item(ptdata->data->tree,ptdata->parent,signal_pt_data[SIGNAL_PARAM_ADD].node);
  if (!parent)
    return;

  paramnode = gtk_ctree_add_node(ptdata->data->tree,parent,"Parameter");
  if (!paramnode)
    return;

  gtk_ctree_node_set_row_data(ptdata->data->tree,paramnode,(gpointer)param);
  gtk_ctree_expand(ptdata->data->tree,paramnode);

  pt_data = gwf_treedata_register_ptdata(ptdata->data,param_pt_data,LAST_PARAM_NODE,paramnode,project);
  if (!pt_data)
    return;

  gwf_projtree_insert_string_by_ptdata(pt_data[PARAM_NAME],name);
  gwf_projtree_insert_string_by_ptdata(pt_data[PARAM_TYPE],gwf_project_ofrsig_param_get_type(project,signame,name));
  gwf_projtree_insert_string_by_ptdata(pt_data[PARAM_GTKTYPE],gtk_type_to_string(gwf_project_ofrsig_param_get_gtktype(project,signame,name)));
}

static void
signal_tree_param_removed_cb(GwfProject *project,gchar *signame,gchar *name,GwfPTData *ptdata)
{
  sigparam	*param;
  GtkCTreeNode	*paramnode;

  if (!signal_tree_check_current_signal(project,signame,ptdata))
    return;

  param = gwf_project_ofrsig_param_find(project,signame,name);
  g_return_if_fail(param != NULL);

  paramnode = gtk_ctree_find_by_row_data(ptdata->data->tree,ptdata->parent,(gpointer)param);
  g_return_if_fail(paramnode != NULL);

  gwf_projtree_remove_node_ptdata(ptdata->data,paramnode);
}






static void
signal_tree_retval_changed_cb (GwfProject *project,gchar *name,gpointer user_data)
{
  if (signal_tree_check_current_signal(project,name,(GwfPTData*)user_data))
    gwf_projtree_node_string_changed_cb(project,gwf_project_ofrsig_get_retval(project,name),user_data);
}

static void
signal_tree_rvtype_changed_cb (GwfProject *project,gchar *name,gpointer user_data)
{
  if (signal_tree_check_current_signal(project,name,(GwfPTData*)user_data))
    gwf_projtree_node_string_changed_cb(project,gtk_type_to_string(gwf_project_ofrsig_get_rvtype(project,name)),user_data);
}

static void
signal_tree_name_changed_cb (GwfProject *project,gchar *name,gpointer user_data)
{
  if (signal_tree_check_current_signal(project,name,(GwfPTData*)user_data))
    gwf_projtree_node_string_changed_cb(project,name,user_data);
}

static gint
signal_tree_check_current_signal(GwfProject *project,gchar *signame,GwfPTData *ptdata)
{
  offersig	*sig;
  GtkCTreeNode	*signal;

  sig = gwf_project_ofrsig_find(project,signame);
  g_return_val_if_fail(sig != NULL,0);

  signal = gtk_ctree_find_by_row_data(ptdata->data->tree,ptdata->parent,(gpointer)sig);
  return (signal == ptdata->parent);
}

static void
signal_tree_param_name_changed_cb (GwfProject *project,gchar *signame,gchar *name,gpointer user_data)
{
  if (signal_tree_check_current_param_signal(project,signame,name,(GwfPTData*)user_data))
    gwf_projtree_node_string_changed_cb(project,name,user_data);
}

static void
signal_tree_param_type_changed_cb (GwfProject *project,gchar *signame,gchar *name,gpointer user_data)
{
  if (signal_tree_check_current_param_signal(project,signame,name,(GwfPTData*)user_data))
    gwf_projtree_node_string_changed_cb(project,gwf_project_ofrsig_param_get_type(project,signame,name),user_data);
}

static void
signal_tree_param_gtktype_changed_cb (GwfProject *project,gchar *signame,gchar *name,gpointer user_data)
{
  if (signal_tree_check_current_param_signal(project,signame,name,(GwfPTData*)user_data))
    gwf_projtree_node_string_changed_cb(project,gtk_type_to_string(gwf_project_ofrsig_param_get_gtktype(project,signame,name)),user_data);
}


static gint
signal_tree_check_current_param_signal(GwfProject *project,gchar *signame,gchar *name,GwfPTData *ptdata)
{
  sigparam	*param;
  GtkCTreeNode	*paramnode;

  param = gwf_project_ofrsig_param_find(project,signame,name);
  g_return_val_if_fail(param != NULL,0);

  paramnode = gtk_ctree_find_by_row_data(ptdata->data->tree,ptdata->parent,(gpointer)param);
  return (paramnode == ptdata->parent);
}
